@@ -22,6 +22,7 @@ from functools import lru_cache, reduce
 from inspect import Parameter, Signature, isabstract, isclass, signature
 from types import FunctionType
 from typing import (
+    TYPE_CHECKING,
     Any,
     AnyStr,
     Callable,
@@ -45,8 +46,8 @@ from uuid import UUID
 import attr
 
 from hypothesis._settings import note_deprecation
-from hypothesis.control import cleanup, note
-from hypothesis.errors import InvalidArgument, ResolutionFailed
+from hypothesis.control import cleanup, current_build_context, note
+from hypothesis.errors import InvalidArgument, ResolutionFailed, RewindRecursive
 from hypothesis.internal.cathetus import cathetus
 from hypothesis.internal.charmap import as_general_categories
 from hypothesis.internal.compat import (
@@ -63,6 +64,7 @@ from hypothesis.internal.conjecture.utils import (
     integer_range,
 )
 from hypothesis.internal.entropy import get_seeder_and_restorer
+from hypothesis.internal.floats import float_of
 from hypothesis.internal.reflection import (
     define_function_signature,
     get_pretty_function_description,
@@ -115,19 +117,22 @@ from hypothesis.strategies._internal.strings import (
 )
 from hypothesis.strategies._internal.utils import cacheable, defines_strategy
 from hypothesis.utils.conventions import not_set
+from hypothesis.vendor.pretty import RepresentationPrinter
 
-if sys.version_info >= (3, 10):  # pragma: no cover
+if sys.version_info >= (3, 10):
     from types import EllipsisType as EllipsisType
 elif typing.TYPE_CHECKING:  # pragma: no cover
     from builtins import ellipsis as EllipsisType
 else:
-    EllipsisType = type(Ellipsis)
+    EllipsisType = type(Ellipsis)  # pragma: no cover
 
 
-try:
+if sys.version_info >= (3, 8):
     from typing import Protocol
-except ImportError:  # < py3.8
-    Protocol = object  # type: ignore[assignment]
+elif TYPE_CHECKING:
+    from typing_extensions import Protocol
+else:  # pragma: no cover
+    Protocol = object
 
 
 @cacheable
@@ -829,11 +834,10 @@ class BuildsStrategy(SearchStrategy):
         self.kwargs = kwargs
 
     def do_draw(self, data):
+        args = [data.draw(a) for a in self.args]
+        kwargs = {k: data.draw(v) for k, v in self.kwargs.items()}
         try:
-            return self.target(
-                *(data.draw(a) for a in self.args),
-                **{k: data.draw(v) for k, v in self.kwargs.items()},
-            )
+            obj = self.target(*args, **kwargs)
         except TypeError as err:
             if (
                 isinstance(self.target, type)
@@ -863,6 +867,9 @@ class BuildsStrategy(SearchStrategy):
                 ) from err
             raise
 
+        current_build_context().record_call(obj, self.target, args, kwargs)
+        return obj
+
     def validate(self):
         tuples(*self.args).validate()
         fixed_dictionaries(self.kwargs).validate()
@@ -899,7 +906,7 @@ def builds(
 
     If the callable is a class defined with :pypi:`attrs`, missing required
     arguments will be inferred from the attribute on a best-effort basis,
-    e.g. by checking :ref:`attrs standard validators <attrs:api_validators>`.
+    e.g. by checking :ref:`attrs standard validators <attrs:api-validators>`.
     Dataclasses are handled natively by the inference from type hints.
 
     Examples from this strategy shrink by shrinking the argument values to
@@ -957,7 +964,7 @@ def builds(
     return BuildsStrategy(target, args, kwargs)
 
 
-if sys.version_info[:2] >= (3, 8):  # pragma: no branch
+if sys.version_info[:2] >= (3, 8):
     # See notes above definition - this signature is compatible and better
     # matches the semantics of the function.  Great for documentation!
     sig = signature(builds)
@@ -981,7 +988,7 @@ if sys.version_info[:2] >= (3, 8):  # pragma: no branch
 
 @cacheable
 @defines_strategy(never_lazy=True)
-def from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
+def from_type(thing: Type[Ex], *, force_defer: bool = False) -> SearchStrategy[Ex]:
     """Looks up the appropriate search strategy for the given type.
 
     ``from_type`` is used internally to fill in missing arguments to
@@ -1033,22 +1040,24 @@ def from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
     This is useful when writing tests which check that invalid input is
     rejected in a certain way.
     """
+    if not force_defer:
+        try:
+            return _from_type(thing, [])
+        except Exception:
+            pass
     # This tricky little dance is because we want to show the repr of the actual
     # underlying strategy wherever possible, as a form of user education, but
     # would prefer to fall back to the default "from_type(...)" repr instead of
     # "deferred(...)" for recursive types or invalid arguments.
-    try:
-        return _from_type(thing)
-    except Exception:
-        return LazyStrategy(
-            lambda thing: deferred(lambda: _from_type(thing)),
-            (thing,),
-            {},
-            force_repr=f"from_type({thing!r})",
-        )
+    return LazyStrategy(
+        lambda thing: deferred(lambda: _from_type(thing, [])),
+        (thing,),
+        {},
+        force_repr=f"from_type({thing!r})",
+    )
 
 
-def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
+def _from_type(thing: Type[Ex], recurse_guard: List[Type[Ex]]) -> SearchStrategy[Ex]:
     # TODO: We would like to move this to the top level, but pending some major
     # refactoring it's hard to do without creating circular imports.
     from hypothesis.strategies._internal import types
@@ -1070,17 +1079,31 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
             raise ResolutionFailed(f"Error: {thing!r} resolved to an empty strategy")
         return strategy
 
+    def defer_recursion(thing, producer):
+        """Returns the result of producer, or ... if recursion on thing is encountered"""
+        if thing in recurse_guard:
+            raise RewindRecursive(thing)
+        recurse_guard.append(thing)
+        try:
+            return producer()
+        except RewindRecursive as rr:
+            if rr.target != thing:
+                raise
+            return ...  # defer resolution
+        finally:
+            recurse_guard.pop()
+
     if not isinstance(thing, type):
         if types.is_a_new_type(thing):
             # Check if we have an explicitly registered strategy for this thing,
             # resolve it so, and otherwise resolve as for the base type.
             if thing in types._global_type_lookup:
                 return as_strategy(types._global_type_lookup[thing], thing)
-            return from_type(thing.__supertype__)
+            return _from_type(thing.__supertype__, recurse_guard)
         # Unions are not instances of `type` - but we still want to resolve them!
         if types.is_a_union(thing):
             args = sorted(thing.__args__, key=types.type_sorting_key)
-            return one_of([from_type(t) for t in args])
+            return one_of([_from_type(t, recurse_guard) for t in args])
     if not types.is_a_type(thing):
         if isinstance(thing, str):
             # See https://github.com/HypothesisWorks/hypothesis/issues/3016
@@ -1135,7 +1158,9 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
                     raise InvalidArgument(
                         f"`{k}: {v.__name__}` is not a valid type annotation"
                     ) from None
-            anns[k] = from_type(v)
+            anns[k] = defer_recursion(v, lambda: _from_type(v, recurse_guard))
+            if anns[k] is ...:
+                anns[k] = from_type(v, force_defer=True)
         if (
             (not anns)
             and thing.__annotations__
@@ -1211,7 +1236,10 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
                 and p.default is not Parameter.empty
                 and p.kind in (Parameter.POSITIONAL_OR_KEYWORD, Parameter.KEYWORD_ONLY)
             ):
-                kwargs[k] = just(p.default) | _from_type(hints[k])
+                kwargs[k] = defer_recursion(
+                    hints[k],
+                    lambda: just(p.default) | _from_type(hints[k], recurse_guard),
+                )
         return builds(thing, **kwargs)
     # And if it's an abstract type, we'll resolve to a union of subclasses instead.
     subclasses = thing.__subclasses__()
@@ -1223,13 +1251,13 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
     subclass_strategies = nothing()
     for sc in subclasses:
         try:
-            subclass_strategies |= _from_type(sc)
+            subclass_strategies |= _from_type(sc, recurse_guard)
         except Exception:
             pass
     if subclass_strategies.is_empty:
         # We're unable to resolve subclasses now, but we might be able to later -
         # so we'll just go back to the mixed distribution.
-        return sampled_from(subclasses).flatmap(from_type)
+        return sampled_from(subclasses).flatmap(lambda t: _from_type(t, recurse_guard))
     return subclass_strategies
 
 
@@ -1555,7 +1583,7 @@ def _composite(f):
     if not is_first_param_referenced_in_function(f):
         note_deprecation(
             "There is no reason to use @st.composite on a function which "
-            + "does not call the provided draw() function internally.",
+            "does not call the provided draw() function internally.",
             since="2022-07-17",
             has_codemod=False,
         )
@@ -1624,8 +1652,10 @@ def complex_numbers(
     allow_infinity: Optional[bool] = None,
     allow_nan: Optional[bool] = None,
     allow_subnormal: bool = True,
+    width: int = 128,
 ) -> SearchStrategy[complex]:
-    """Returns a strategy that generates complex numbers.
+    """Returns a strategy that generates :class:`~python:complex`
+    numbers.
 
     This strategy draws complex numbers with constrained magnitudes.
     The ``min_magnitude`` and ``max_magnitude`` parameters should be
@@ -1636,13 +1666,22 @@ def complex_numbers(
     is an error to enable ``allow_nan``.  If ``max_magnitude`` is finite,
     it is an error to enable ``allow_infinity``.
 
-    ``allow_subnormal`` is applied to each part of the complex number
-    separately, as for :func:`~hypothesis.strategies.floats`.
+    ``allow_infinity``, ``allow_nan``, and ``allow_subnormal`` are
+    applied to each part of the complex number separately, as for
+    :func:`~hypothesis.strategies.floats`.
 
     The magnitude constraints are respected up to a relative error
     of (around) floating-point epsilon, due to implementation via
     the system ``sqrt`` function.
 
+    The ``width`` argument specifies the maximum number of bits of precision
+    required to represent the entire generated complex number.
+    Valid values are 32, 64 or 128, which correspond to the real and imaginary
+    components each having width 16, 32 or 64, respectively.
+    Passing ``width=64`` will still use the builtin 128-bit
+    :class:`~python:complex` class, but always for values which can be
+    exactly represented as two 32-bit floats.
+
     Examples from this strategy shrink by shrinking their real and
     imaginary parts, as :func:`~hypothesis.strategies.floats`.
 
@@ -1671,8 +1710,15 @@ def complex_numbers(
             f"Cannot have allow_nan={allow_nan!r}, min_magnitude={min_magnitude!r} "
             f"max_magnitude={max_magnitude!r}"
         )
-
     check_type(bool, allow_subnormal, "allow_subnormal")
+    if width not in (32, 64, 128):
+        raise InvalidArgument(
+            f"width={width!r}, but must be 32, 64 or 128 (other complex dtypes "
+            "such as complex192 or complex256 are not supported)"
+            # For numpy, these types would be supported (but not by CPython):
+            # https://numpy.org/doc/stable/reference/arrays.scalars.html#complex-floating-point-types
+        )
+    component_width = width // 2
     allow_kw = {
         "allow_nan": allow_nan,
         "allow_infinity": allow_infinity,
@@ -1680,6 +1726,7 @@ def complex_numbers(
         # then allow_subnormal=True would be an error with the min_value to the floats()
         # strategy for the real part.  We therefore replace True with None.
         "allow_subnormal": None if allow_subnormal else allow_subnormal,
+        "width": component_width,
     }
 
     if min_magnitude == 0 and max_magnitude is None:
@@ -1690,19 +1737,32 @@ def complex_numbers(
 
     @composite
     def constrained_complex(draw):
+        # We downcast drawn floats to the desired (component) width so we
+        # guarantee the resulting complex values are representable. Note
+        # truncating the mantissa bits with float_of() cannot increase the
+        # magnitude of a float, so we are guaranteed to stay within the allowed
+        # range. See https://github.com/HypothesisWorks/hypothesis/issues/3573
+
         # Draw the imaginary part, and determine the maximum real part given
         # this and the max_magnitude
         if max_magnitude is None:
             zi = draw(floats(**allow_kw))
             rmax = None
         else:
-            zi = draw(floats(-max_magnitude, max_magnitude, **allow_kw))
-            rmax = cathetus(max_magnitude, zi)
+            zi = draw(
+                floats(
+                    -float_of(max_magnitude, component_width),
+                    float_of(max_magnitude, component_width),
+                    **allow_kw,
+                )
+            )
+            rmax = float_of(cathetus(max_magnitude, zi), component_width)
         # Draw the real part from the allowed range given the imaginary part
         if min_magnitude == 0 or math.fabs(zi) >= min_magnitude:
             zr = draw(floats(None if rmax is None else -rmax, rmax, **allow_kw))
         else:
-            zr = draw(floats(cathetus(min_magnitude, zi), rmax, **allow_kw))
+            rmin = float_of(cathetus(min_magnitude, zi), component_width)
+            zr = draw(floats(rmin, rmax, **allow_kw))
         # Order of conditions carefully tuned so that for a given pair of
         # magnitude arguments, we always either draw or do not draw the bool
         # (crucial for good shrinking behaviour) but only invert when needed.
@@ -1831,10 +1891,11 @@ class DataObject:
         check_strategy(strategy, "strategy")
         result = self.conjecture_data.draw(strategy)
         self.count += 1
-        if label is not None:
-            note(f"Draw {self.count} ({label}): {result!r}")
-        else:
-            note(f"Draw {self.count}: {result!r}")
+        printer = RepresentationPrinter(context=current_build_context())
+        printer.text(f"Draw {self.count}")
+        printer.text(": " if label is None else f" ({label}): ")
+        printer.pretty(result)
+        note(printer.getvalue())
         return result
 
 
@@ -1986,21 +2047,30 @@ def deferred(definition: Callable[[], SearchStrategy[Ex]]) -> SearchStrategy[Ex]
     return DeferredStrategy(definition)
 
 
+def domains():
+    import hypothesis.provisional
+
+    return hypothesis.provisional.domains()
+
+
 @defines_strategy(force_reusable_values=True)
-def emails() -> SearchStrategy[str]:
+def emails(
+    *, domains: SearchStrategy[str] = LazyStrategy(domains, (), {})
+) -> SearchStrategy[str]:
     """A strategy for generating email addresses as unicode strings. The
     address format is specified in :rfc:`5322#section-3.4.1`. Values shrink
     towards shorter local-parts and host domains.
 
+    If ``domains`` is given then it must be a strategy that generates domain
+    names for the emails, defaulting to :func:`~hypothesis.provisional.domains`.
+
     This strategy is useful for generating "user data" for tests, as
     mishandling of email addresses is a common source of bugs.
     """
-    from hypothesis.provisional import domains
-
     local_chars = string.ascii_letters + string.digits + "!#$%&'*+-/=^_`{|}~"
     local_part = text(local_chars, min_size=1, max_size=64)
     # TODO: include dot-atoms, quoted strings, escaped chars, etc in local part
-    return builds("{}@{}".format, local_part, domains()).filter(
+    return builds("{}@{}".format, local_part, domains).filter(
         lambda addr: len(addr) <= 254
     )
 
