@@ -12,7 +12,7 @@ import datetime as dt
 import os.path
 from calendar import monthrange
 from functools import lru_cache
-from importlib.resources import is_resource
+from importlib import resources
 from typing import Optional
 
 from hypothesis.errors import InvalidArgument
@@ -156,7 +156,7 @@ class DatetimeStrategy(SearchStrategy):
 
         # If we happened to end up with a disallowed imaginary time, reject it.
         if (not self.allow_imaginary) and datetime_does_not_exist(result):
-            data.mark_invalid()
+            data.mark_invalid("nonexistent datetime")
         return result
 
     def draw_naive_datetime_and_combine(self, data, tz):
@@ -165,8 +165,7 @@ class DatetimeStrategy(SearchStrategy):
             return replace_tzinfo(dt.datetime(**result), timezone=tz)
         except (ValueError, OverflowError):
             msg = "Failed to draw a datetime between %r and %r with timezone from %r."
-            data.note_event(msg % (self.min_value, self.max_value, self.tz_strat))
-            data.mark_invalid()
+            data.mark_invalid(msg % (self.min_value, self.max_value, self.tz_strat))
 
 
 @defines_strategy(force_reusable_values=True)
@@ -186,9 +185,14 @@ def datetimes(
 
     ``timezones`` must be a strategy that generates either ``None``, for naive
     datetimes, or :class:`~python:datetime.tzinfo` objects for 'aware' datetimes.
-    You can construct your own, though we recommend using the :pypi:`dateutil
-    <python-dateutil>` package and :func:`hypothesis.extra.dateutil.timezones`
-    strategy, and also provide :func:`hypothesis.extra.pytz.timezones`.
+    You can construct your own, though we recommend using one of these built-in
+    strategies:
+
+    * with Python 3.9 or newer or :pypi:`backports.zoneinfo`:
+      :func:`hypothesis.strategies.timezones`;
+    * with :pypi:`dateutil <python-dateutil>`:
+      :func:`hypothesis.extra.dateutil.timezones`; or
+    * with :pypi:`pytz`: :func:`hypothesis.extra.pytz.timezones`.
 
     You may pass ``allow_imaginary=False`` to filter out "imaginary" datetimes
     which did not (or will not) occur due to daylight savings, leap seconds,
@@ -348,7 +352,12 @@ def _valid_key_cacheable(tzpath, key):
         *package_loc, resource_name = key.split("/")
         package = "tzdata.zoneinfo." + ".".join(package_loc)
         try:
-            return is_resource(package, resource_name)
+            try:
+                traversable = resources.files(package) / resource_name
+                return traversable.exists()
+            except (AttributeError, ValueError):
+                # .files() was added in Python 3.9
+                return resources.is_resource(package, resource_name)
         except ModuleNotFoundError:
             return False
 
