@@ -21,6 +21,7 @@ from typing import (
     Sequence,
     TypeVar,
     Union,
+    cast,
     overload,
 )
 
@@ -28,6 +29,7 @@ from hypothesis._settings import HealthCheck, Phase, Verbosity, settings
 from hypothesis.control import _current_build_context, assume
 from hypothesis.errors import (
     HypothesisException,
+    HypothesisWarning,
     InvalidArgument,
     NonInteractiveExampleWarning,
     UnsatisfiedAssumption,
@@ -287,6 +289,7 @@ class SearchStrategy(Generic[Ex]):
                 "it performs better, saves and replays failures to avoid flakiness, "
                 "and reports minimal examples. (strategy: %r)" % (self,),
                 NonInteractiveExampleWarning,
+                stacklevel=2,
             )
 
         context = _current_build_context.value
@@ -326,7 +329,7 @@ class SearchStrategy(Generic[Ex]):
             deadline=None,
             verbosity=Verbosity.quiet,
             phases=(Phase.generate,),
-            suppress_health_check=HealthCheck.all(),
+            suppress_health_check=list(HealthCheck),
         )
         def example_generating_inner_function(ex):
             self.__examples.append(ex)
@@ -394,6 +397,14 @@ class SearchStrategy(Generic[Ex]):
             raise ValueError(f"Cannot | a SearchStrategy with {other!r}")
         return OneOfStrategy((self, other))
 
+    def __bool__(self) -> bool:
+        warnings.warn(
+            f"bool({self!r}) is always True, did you mean to draw a value?",
+            HypothesisWarning,
+            stacklevel=2,
+        )
+        return True
+
     def validate(self) -> None:
         """Throw an exception if the strategy is not valid.
 
@@ -424,13 +435,13 @@ class SearchStrategy(Generic[Ex]):
         return result
 
     @property
-    def label(self):
+    def label(self) -> int:
         if self.__label is calculating:
             return 0
         if self.__label is None:
             self.__label = calculating
             self.__label = self.calc_label()
-        return self.__label
+        return cast(int, self.__label)
 
     def calc_label(self):
         return self.class_label
@@ -518,8 +529,7 @@ class SampledFromStrategy(SearchStrategy):
     def do_draw(self, data):
         result = self.do_filtered_draw(data)
         if result is filter_not_satisfied:
-            data.note_event(f"Aborted test because unable to satisfy {self!r}")
-            data.mark_invalid()
+            data.mark_invalid(f"Aborted test because unable to satisfy {self!r}")
         return result
 
     def get_element(self, i):
@@ -587,7 +597,7 @@ class SampledFromStrategy(SearchStrategy):
         return filter_not_satisfied
 
 
-class OneOfStrategy(SearchStrategy):
+class OneOfStrategy(SearchStrategy[Ex]):
     """Implements a union of strategies. Given a number of strategies this
     generates values which could have come from any of them.
 
@@ -681,57 +691,61 @@ class OneOfStrategy(SearchStrategy):
 
 
 @overload
-def one_of(args: Sequence[SearchStrategy[Any]]) -> SearchStrategy[Any]:
-    raise NotImplementedError
+def one_of(
+    __args: Sequence[SearchStrategy[Any]],
+) -> SearchStrategy[Any]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
-def one_of(a1: SearchStrategy[Ex]) -> SearchStrategy[Ex]:
-    raise NotImplementedError
+def one_of(__a1: SearchStrategy[Ex]) -> SearchStrategy[Ex]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
 def one_of(
-    a1: SearchStrategy[Ex], a2: SearchStrategy[T]
-) -> SearchStrategy[Union[Ex, T]]:
-    raise NotImplementedError
+    __a1: SearchStrategy[Ex], __a2: SearchStrategy[T]
+) -> SearchStrategy[Union[Ex, T]]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
 def one_of(
-    a1: SearchStrategy[Ex], a2: SearchStrategy[T], a3: SearchStrategy[T3]
-) -> SearchStrategy[Union[Ex, T, T3]]:
-    raise NotImplementedError
+    __a1: SearchStrategy[Ex], __a2: SearchStrategy[T], __a3: SearchStrategy[T3]
+) -> SearchStrategy[Union[Ex, T, T3]]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
 def one_of(
-    a1: SearchStrategy[Ex],
-    a2: SearchStrategy[T],
-    a3: SearchStrategy[T3],
-    a4: SearchStrategy[T4],
-) -> SearchStrategy[Union[Ex, T, T3, T4]]:
-    raise NotImplementedError
+    __a1: SearchStrategy[Ex],
+    __a2: SearchStrategy[T],
+    __a3: SearchStrategy[T3],
+    __a4: SearchStrategy[T4],
+) -> SearchStrategy[Union[Ex, T, T3, T4]]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
 def one_of(
-    a1: SearchStrategy[Ex],
-    a2: SearchStrategy[T],
-    a3: SearchStrategy[T3],
-    a4: SearchStrategy[T4],
-    a5: SearchStrategy[T5],
-) -> SearchStrategy[Union[Ex, T, T3, T4, T5]]:
-    raise NotImplementedError
+    __a1: SearchStrategy[Ex],
+    __a2: SearchStrategy[T],
+    __a3: SearchStrategy[T3],
+    __a4: SearchStrategy[T4],
+    __a5: SearchStrategy[T5],
+) -> SearchStrategy[Union[Ex, T, T3, T4, T5]]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
-def one_of(*args: SearchStrategy[Any]) -> SearchStrategy[Any]:
-    raise NotImplementedError
+def one_of(*args: SearchStrategy[Any]) -> SearchStrategy[Any]:  # pragma: no cover
+    ...
 
 
 @defines_strategy(never_lazy=True)
-def one_of(*args):  # noqa: F811
+def one_of(
+    *args: Union[Sequence[SearchStrategy[Any]], SearchStrategy[Any]]
+) -> SearchStrategy[Any]:  # noqa: F811
     # Mypy workaround alert:  Any is too loose above; the return parameter
     # should be the union of the input parameters.  Unfortunately, Mypy <=0.600
     # raises errors due to incompatible inputs instead.  See #1270 for links.
@@ -775,7 +789,7 @@ def one_of(*args):  # noqa: F811
     return OneOfStrategy(args)
 
 
-class MappedSearchStrategy(SearchStrategy):
+class MappedSearchStrategy(SearchStrategy[Ex]):
     """A strategy which is defined purely by conversion to and from another
     strategy.
 
@@ -810,23 +824,22 @@ class MappedSearchStrategy(SearchStrategy):
         into a value suitable for outputting from this strategy."""
         raise NotImplementedError(f"{self.__class__.__name__}.pack()")
 
-    def do_draw(self, data: ConjectureData) -> Ex:
+    def do_draw(self, data: ConjectureData) -> Any:
         with warnings.catch_warnings():
             if isinstance(self.pack, type) and issubclass(
                 self.pack, (abc.Mapping, abc.Set)
             ):
                 warnings.simplefilter("ignore", BytesWarning)
             for _ in range(3):
-                i = data.index
                 try:
                     data.start_example(MAPPED_SEARCH_STRATEGY_DO_DRAW_LABEL)
-                    result = self.pack(data.draw(self.mapped_strategy))
+                    x = data.draw(self.mapped_strategy)
+                    result = self.pack(x)  # type: ignore
                     data.stop_example()
+                    _current_build_context.value.record_call(result, self.pack, [x], {})
                     return result
                 except UnsatisfiedAssumption:
                     data.stop_example(discard=True)
-                    if data.index == i:
-                        raise
         raise UnsatisfiedAssumption()
 
     @property
@@ -840,7 +853,7 @@ class MappedSearchStrategy(SearchStrategy):
 filter_not_satisfied = UniqueIdentifier("filter not satisfied")
 
 
-class FilteredStrategy(SearchStrategy):
+class FilteredStrategy(SearchStrategy[Ex]):
     def __init__(self, strategy, conditions):
         super().__init__()
         if isinstance(strategy, FilteredStrategy):
@@ -930,8 +943,7 @@ class FilteredStrategy(SearchStrategy):
         if result is not filter_not_satisfied:
             return result
 
-        data.note_event(f"Aborted test because unable to satisfy {self!r}")
-        data.mark_invalid()
+        data.mark_invalid(f"Aborted test because unable to satisfy {self!r}")
         raise NotImplementedError("Unreachable, for Mypy")
 
     def note_retried(self, data):
